/*#############################################################
File Name       : ex1_SWOViewer_HelloWorld.c
Author          : Grant Phillips
Date Modified   : 13/09/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description     : Example program that uses Trace Events to 
									write "Hello World!" via the ST-Link programmer 
									to the ST-Link Utility's SWO Viewer and then 
									display a count from 0 to 255 repeatedly. The
									count value is also written to the 8 USER LEDs.
									Visit the following link for more information
									regarding the SWO Viewer:
									http://controlsoft.nmmu.ac.za/STM32F4-Discovery
									-Board/Example-programs/Serial-Wire-Viewer-(SWV)        

Requirements    : * STM32F4-Discovery Board
              
Circuit         : * A wire link between PB3 and pin6 of the SWD
										connector (CN2)
										OR
										solder the solder bridge SB12 closed underneath
										the board
									* 8 LEDs connected to PD15 - PD8

See the STM32F4-Discovery User Manual (UM1472) for the block 
diagram of the STM32F407VGT6 processor (p12), a summary of
the GPIO connections (p20-29) and the schematic diagram (p31-36)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "STM32F4_Defaults.c"																		//library for the defaults of a blank project in Keil uVision

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void InitLEDs(void);																					  //prototype for the user function to initialize the USER LEDs

/* #global variables -----------------------------------------*/



int main(void)
{
	RCC_ClocksTypeDef 			RCC_Clocks;														//structure used for setting up the SysTick Interrupt
	
	uint8_t count=0;																							//8-bit integer variable to hold the count value (0-255)
  
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
	
	InitLEDs();																									  //initialize the USER LEDs for this application	
	
	//print a message to the SWO Viewer (make sure the fputc function is 
	//retargeted as shown by the fputc function just after main()
	printf("Hallo World!\n");	
	
	/* Main program loop */
  while (1)
  {
		printf("\n%2X", count); 																		//print a newline and integer value (in HEX)
		
		GPIO_Write(GPIOD, count<<8);																//shift the value 8 spaces to the left, because we are writing it to the high byte (PD15 - PD8)
		
		count = count + 1;
		
    Delay(250);
  }	
}



/*
Retarget the C library printf function to the SWO Viewer.
Overwrites int fputc(int ch, FILE *f) function used by printf.
*/								
int fputc(int ch, FILE *f)
{
	return(ITM_SendChar(ch));
}



void InitLEDs(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;										//structure used for setting up a GPIO port
	
	/* GPIOD Periph clock enable */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);					//enable the AHB1 bus to use GPIOD
  
  /* Configure PD15 - PD8 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15 | GPIO_Pin_14 | GPIO_Pin_13 | GPIO_Pin_12 | GPIO_Pin_11 | GPIO_Pin_10 | GPIO_Pin_9 | GPIO_Pin_8;	//which pins to setup, seperated by |
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;								
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;							
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOD, &GPIO_InitStructure);											                                 
}
